@@ -48,7 +48,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
+#ifndef WIN32
 #include <unistd.h>
+#endif
 #include <string.h>
 #include "rsa.h"
 #include "envelope.h"
@@ -86,6 +88,8 @@
  * -keyform	- key file format.
  * -newkey	- make a key and a request.
  * -x509	- output a self signed X509 structure instead.
+ * -asn1-kludge	- output new certificate request in a format that some CA's
+ *		  require.  This format is wrong
  */
 
 #ifndef NOPROTO
@@ -118,7 +122,7 @@ char **argv;
 	int i,badops=0,new=0,newkey= -1;
 	FILE *in=NULL,*out=NULL;
 	int informat,outformat,verify=0,noout=0,text=0,keyform=FORMAT_PEM;
-	int nodes=0;
+	int nodes=0,kludge=0;
 	char *infile,*outfile,*prog,*keyfile=NULL,*template=NULL,*keyout=NULL;
 	EVP_CIPHER *cipher=EVP_des_ede3_cbc;
 	char *p;
@@ -191,6 +195,8 @@ char **argv;
 			text=1;
 		else if (strcmp(*argv,"-x509") == 0)
 			x509=1;
+		else if (strcmp(*argv,"-asn1-kludge") == 0)
+			kludge=1;
 		else if (strcmp(*argv,"-days") == 0)
 			{
 			if (--argc < 1) goto bad;
@@ -227,7 +233,9 @@ bad:
 		fprintf(stderr," -config file   request templace file.\n");
 		fprintf(stderr," -new           new request.\n");
 		fprintf(stderr," -x509          output a x509 structure instead of a cert. req.\n");
-		fprintf(stderr," -days          number of days a x509 self-signed certificate is valid for.\n");
+		fprintf(stderr," -days          number of days a x509 generated by -x509 is valid for.\n");
+		fprintf(stderr," -asn1-kludge   Output the 'request' in a format that is wrong but some CA's\n");
+		fprintf(stderr,"                have been reported as requiring\n");
 		goto end;
 		}
 
@@ -282,7 +290,13 @@ bad:
 		if ((randfile=CONF_get_string(config,SECTION,"RANDFILE")) == NULL)
 			randfile=RAND_file_name(buffer,200);
 		if ((randfile == NULL) || !RAND_load_file(randfile,1024L*1024L))
+			{
 			fprintf(stderr,"unable to load 'random state'\n");
+			fprintf(stderr,"What this means is that the random number generator has not been seeded\n");
+			fprintf(stderr,"with much random data.\n");
+			fprintf(stderr,"Consider setting the RANDFILE environment variable to point at a file that\n");
+			fprintf(stderr,"'random' data can be kept in.\n");
+			}
 		if (newkey <= 0)
 			{
 			newkey=CONF_get_number(config,SECTION,BITS);
@@ -358,6 +372,10 @@ loop:
 
 	if (!new)
 		{
+		/* Since we are using a pre-existing certificate
+		 * request, the kludge 'format' info should not be
+		 * changed. */
+		kludge= -1;
 		if (infile == NULL)
 			in=stdin;
 		else
@@ -372,10 +390,6 @@ loop:
 
 		if	(informat == FORMAT_ASN1)
 			req=d2i_X509_REQ_fp(in,NULL);
-#ifdef F2I
-		else if (informat == FORMAT_TEXT)
-			i=f2i_X509_REQ(in,req);
-#endif
 		else if (informat == FORMAT_PEM)
 			req=PEM_read_X509_REQ(in,NULL,NULL);
 		else
@@ -403,6 +417,8 @@ loop:
 				}
 
 			i=make_REQ(req,rsa,!x509);
+			if (kludge >= 0)
+				req->req_info->req_kludge=kludge;
 			if (!i)
 				{
 				fprintf(stderr,"problems making Certificate Request\n");
@@ -618,7 +634,7 @@ int attribs;
 start:		for (;;)
 			{
 			i++;
-			if (sk_num(sk) <= i) break;
+			if ((int)sk_num(sk) <= i) break;
 
 			v=(CONF_VALUE *)sk_value(sk,i);
 			type=v->name;
@@ -661,7 +677,7 @@ start:		for (;;)
 start2:			for (;;)
 				{
 				i++;
-				if ((attr == NULL) || (sk_num(attr) <= i))
+				if ((attr == NULL) || ((int)sk_num(attr) <= i))
 					break;
 
 				v=(CONF_VALUE *)sk_value(attr,i);
