@@ -48,7 +48,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef WIN32
 #include <unistd.h>
+#endif
+#include "lhash.h"
 #include "s_net.h"
 #include "x509.h"
 #include "ssl.h"
@@ -62,11 +65,13 @@ static int body(char *hostname, int s);
 static void close_accept_socket(void );
 static void usage(void);
 static int init_ssl_connection(SSL *s);
+static void print_stats(FILE *fp,SSL_CTX *ctx);
 #else
 static int body();
 static void close_accept_socket();
 static void usage();
 static int init_ssl_connection();
+static void print_stats();
 #endif
 
 #define BUFSIZZ	10*1024
@@ -78,6 +83,7 @@ static int accept_socket=-1;
 extern int verify_depth;
 extern int verify_error;
 
+static char *cipher=NULL;
 int verify=SSL_VERIFY_NONE;
 char *cert_file=TEST_CERT,*key_file=NULL;
 #ifdef FIONBIO
@@ -88,7 +94,7 @@ static SSL_CTX *ctx=NULL;
 
 static void usage()
 	{
-	fprintf(stderr,"usage: server args\n");
+	fprintf(stderr,"usage: s_server [args ...]\n");
 	fprintf(stderr,"\n");
 	fprintf(stderr," -port arg     - port to connect to (default is %d\n",PORT);
 	fprintf(stderr," -verify arg   - turn on peer certificate verification\n");
@@ -102,6 +108,11 @@ static void usage()
 #endif
 	fprintf(stderr," -CApath arg   - PEM format directory of CA's\n");
 	fprintf(stderr," -CAfile arg   - PEM format file of CA's\n");
+	fprintf(stderr," -cipher       - the only ciphers to use, ':' seperated list of the following\n");
+	fprintf(stderr,"                  NULL-MD5     RC4-MD5      EXP-RC4-MD5 \n");
+        fprintf(stderr,"                 IDEA-CBC-MD5 RC2-CBC-MD5  EXP-RC2-CBC-MD5\n");
+        fprintf(stderr,"                 DES-CBC-MD5  DES-CBC-SHA  DES-CBC3-MD5\n");
+        fprintf(stderr,"                 DES-CBC3-SHA DES-CFB-M1\n");
 	}
 
 int MAIN(argc, argv)
@@ -163,6 +174,11 @@ char *argv[];
 			if (--argc < 1) goto bad;
 			CApath= *(++argv);
 			}
+		else if	(strcmp(*argv,"-cipher") == 0)
+			{
+			if (--argc < 1) goto bad;
+			cipher= *(++argv);
+			}
 		else if	(strcmp(*argv,"-CAfile") == 0)
 			{
 			if (--argc < 1) goto bad;
@@ -188,6 +204,8 @@ bad:
 		goto end;
 		}
 
+	if (cipher == NULL) cipher=getenv("SSL_CIPHER");
+
 	if (cert_file == NULL)
 		{
 		fprintf(stderr,"You must specify a certificate file for the server to use\n");
@@ -204,14 +222,36 @@ bad:
 		goto end;
 		}
 
+	if (cipher != NULL)
+		SSL_CTX_set_cipher_list(ctx,cipher);
 	fprintf(stderr,"ACCEPT\n");
-	do_server(port,NULL,body);
+	do_server(port,&accept_socket,body);
 	ret=0;
 end:
+	print_stats(stderr,ctx);
 	if (ctx != NULL) SSL_CTX_free(ctx);
 	EXIT(ret);
 	}
 
+static void print_stats(fp,ctx)
+FILE *fp;
+SSL_CTX *ctx;
+	{
+	fprintf(fp,"%4ld items in the session cache\n",
+		SSL_CTX_sess_number(ctx));
+	fprintf(fp,"%4ld client connects (SSL_connect())\n",
+		SSL_CTX_sess_connect(ctx));
+	fprintf(fp,"%4ld client connects that finished\n",
+		SSL_CTX_sess_connect_good(ctx));
+	fprintf(fp,"%4ld server connects (SSL_accept())\n",
+		SSL_CTX_sess_accept(ctx));
+	fprintf(fp,"%4ld server connects that finished\n",
+		SSL_CTX_sess_accept_good(ctx));
+	fprintf(fp,"%4ld session cache hits\n",SSL_CTX_sess_hits(ctx));
+	fprintf(fp,"%4ld session cache misses\n",SSL_CTX_sess_misses(ctx));
+	fprintf(fp,"%4ld session cache timeouts\n",SSL_CTX_sess_timeouts(ctx));
+	}
+
 static int body(hostname, s)
 char *hostname;
 int s;
@@ -237,7 +277,7 @@ int s;
 		SSL_set_verify(con,verify,verify_callback);
 		if (!set_cert_stuff(con,cert_file,key_file))
 			{
-			ret=0;
+			ret=-1;
 			goto err;
 			}
 		}
@@ -258,11 +298,15 @@ int s;
 			if ((i <= 0) || (buf[0] == 'Q'))
 				{
 				fprintf(stderr,"DONE\n");
+				shutdown(s,2);
 				close_accept_socket();
-				ret=1;
-				exit(0);
+				ret= -11;
 				goto err;
 				}
+			if (buf[0] == 'S')
+				{
+				print_stats(stderr,SSL_get_SSL_CTX(con));
+				}
 			l=k=0;
 			for (;;)
 				{
@@ -331,8 +375,10 @@ int s;
 			}
 		}
 err:
-/*	if (con != NULL) SSL_free(con); */
-	fprintf(stderr,"CONNECTION CLOSED\nACCEPT\n");
+/*	if (con != NULL) SSL_free(con);  */
+	fprintf(stderr,"CONNECTION CLOSED\n");
+	if (ret >= 0)
+		fprintf(stderr,"ACCEPT\n");
 	return(ret);
 	}
 
@@ -368,6 +414,9 @@ SSL *con;
 			ERR_print_errors(stderr);
 		return(0);
 		}
+
+	PEM_write_SSL_SESSION(stdout,SSL_get_session(con));
+
 	peer=SSL_get_peer_certificate(con);
 	if (peer != NULL)
 		{
