@@ -50,7 +50,9 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#ifndef WIN32
 #include <unistd.h>
+#endif
 #include <string.h>
 
 #include "../crypto/crypto.h"
@@ -60,14 +62,14 @@
 #include "pem.h"
 #include "apps.h"
 #include "s_apps.h"
-#ifdef _SSLWIN
+#ifdef WIN32_STUFF
 #include "winmain.h"
 #include "wintext.h"
 #endif
 
 #define TIMES
 
-#ifdef _SSLWIN
+#ifdef WIN32
 #undef TIMES
 #endif
 
@@ -153,7 +155,6 @@ static char *key_file=NULL;
 static char *CApath=NULL;
 static char *CAfile=NULL;
 static char *cipher=NULL;
-static int reconnect = 0;
 static int verify = SSL_VERIFY_NONE;
 static int maxTime = SECONDS;
 static SSL_CTX *ctx=NULL;
@@ -162,7 +163,7 @@ static SSL_CTX *ctx=NULL;
 static int nbio=0;
 #endif
 
-#ifdef _SSLWIN
+#ifdef WIN32
 static int exitNow = 0;		/* Set when it's time to exit main */
 #endif
 
@@ -172,17 +173,18 @@ static int exitNow = 0;		/* Set when it's time to exit main */
 static void usage()
 {
 	static char umsg[] = "\
--time arg	  - max number of seconds to collect data, default %d\n\
+-time arg     - max number of seconds to collect data, default %d\n\
 -verify arg   - turn on peer certificate verification\n\
 -cert arg     - certificate file to use, PEM format assumed\n\
 -key arg      - RSA file to use, PEM format assumed, in cert file if\n\
                 not specified but cert fill is.\n\
 -CApath arg   - PEM format directory of CA's\n\
 -CAfile arg   - PEM format file of CA's\n\
--reconnect    - Drop and re-make the connection with the same Session-ID\n\
 -cipher       - prefered cipher to use, ':' seperated list of the following\n\
-                RC4-MD5 EXP-RC4-MD5 CBC-DES-MD5 CBC3-DES-MD5 CBC-IDEA-MD5\n\
-                CFB-DES-NULL\n\n";
+                NULL-MD5     RC4-MD5      EXP-RC4-MD5 \n\
+                IDEA-CBC-MD5 RC2-CBC-MD5  EXP-RC2-CBC-MD5\n\
+                DES-CBC-MD5  DES-CBC-SHA  DES-CBC3-MD5\n\
+                DES-CBC3-SHA DES-CFB-M1\n\n";
 
 	printf( "usage: client <args>\n\n" );
 
@@ -244,10 +246,6 @@ char **argv;
 	    if (--argc < 1) goto bad;
 	    key_file= *(++argv);
 
-	} else if( strcmp(*argv,"-reconnect") == 0) {
-
-	    reconnect=1;
-
 	} else if( strcmp(*argv,"-CApath") == 0) {
 
 	    if (--argc < 1) goto bad;
@@ -375,7 +373,7 @@ char **argv;
 	for (;;)
 		{
 		if (finishtime < time(NULL)) break;
-#ifdef _SSLWIN
+#ifdef WIN32_STUFF
 
 		if( flushWinMsgs(0) == -1 ) {
 			EXIT(1);
@@ -422,7 +420,7 @@ char **argv;
 		{
 		if (finishtime < time(NULL)) break;
 
-#ifdef _SSLWIN
+#ifdef WIN32_STUFF
 		if( flushWinMsgs(0) == -1 ) {
 			EXIT(1);
 		}
@@ -482,10 +480,10 @@ SSL *scon;
 	SSL_set_fd(serverCon,s);
 
 	if( scon != NULL )
-		SSL_copy_session_id(serverCon,scon);
+		SSL_set_session(serverCon,SSL_get_session(scon));
 
 /*	SSL_set_verify(serverCon,verify,verify_callback);*/
-	SSL_set_pref_cipher(serverCon,cipher);
+	SSL_CTX_set_cipher_list(ctx,cipher);
 
 	if(!set_cert_stuff(serverCon,cert_file,key_file)) {
 		SHUTDOWN(SSL_get_fd(serverCon));
