@@ -49,17 +49,15 @@
 #include "crypto.h"
 #include "bn.h"
 
-char *BN_version="\0Big Number part of SSLeay v 0.5.1b 23/02/96";
+char *BN_version="Big Number part of SSLeay 0.5.2 29-Apr-1996";
 
 static BN_ULONG data_one=1L;
 static BIGNUM const_one={&data_one,1,1,0};
 BIGNUM *BN_value_one=&const_one;
 
-int BN_num_bits(a)
-BIGNUM *a;
+int BN_num_bits_word(l)
+BN_ULONG l;
 	{
-	int i;
-	BN_ULONG l;
 	static char bits[256]={
 		0,1,2,2,3,3,3,3,4,4,4,4,4,4,4,4,
 		5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
@@ -79,15 +77,6 @@ BIGNUM *a;
 		8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,8,
 		};
 
-	if (a->top == 0) return(0);
-	l=a->d[a->top-1];
-	i=(a->top-1)*BN_BITS2;
-	if (l == 0)
-		{
-		fprintf(stderr,"BAD TOP VALUE\n");
-		abort();
-		}
-
 #ifdef SIXTY_FOUR_BIT
 	if (l & 0xffffffff00000000)
 		{
@@ -95,17 +84,17 @@ BIGNUM *a;
 			{
 			if (l & 0xff00000000000000)
 				{
-				return(i+bits[l>>56]+56);
+				return(bits[l>>56]+56);
 				}
-			else	return(i+bits[l>>48]+48);
+			else	return(bits[l>>48]+48);
 			}
 		else
 			{
 			if (l & 0x0000ff0000000000)
 				{
-				return(i+bits[l>>40]+40);
+				return(bits[l>>40]+40);
 				}
-			else	return(i+bits[l>>32]+32);
+			else	return(bits[l>>32]+32);
 			}
 		}
 	else
@@ -115,19 +104,39 @@ BIGNUM *a;
 		if (l & 0xffff0000L)
 			{
 			if (l & 0xff000000L)
-				return(i+bits[l>>24L]+24);
-			else	return(i+bits[l>>16L]+16);
+				return(bits[l>>24L]+24);
+			else	return(bits[l>>16L]+16);
 			}
 		else
 #endif
 			{
+#if defined(SIXTEEN_BIT) || defined(THIRTY_TWO_BIT) || defined(SIXTY_FOUR_BIT)
 			if (l & 0xff00L)
-				return(i+bits[l>>8]+8);
-			else	return(i+bits[l   ]  );
+				return(bits[l>>8]+8);
+			else	
+#endif
+				return(bits[l   ]  );
 			}
 		}
 	}
 
+int BN_num_bits(a)
+BIGNUM *a;
+	{
+	BN_ULONG l;
+	int i;
+
+	if (a->top == 0) return(0);
+	l=a->d[a->top-1];
+	i=(a->top-1)*BN_BITS2;
+	if (l == 0)
+		{
+		fprintf(stderr,"BAD TOP VALUE\n");
+		abort();
+		}
+	return(i+BN_num_bits_word(l));
+	}
+
 void BN_clear_free(a)
 BIGNUM *a;
 	{
