@@ -50,7 +50,7 @@
 #include <string.h>
 #include "md2.h"
 
-char *MD2_version="\0MD2 part of SSLeay v 0.5.1b 23/02/96";
+char *MD2_version="MD2 part of SSLeay 0.5.2 29-Apr-1996";
 
 /* Implemented from RFC1319 The MD2 Message-Digest Algorithm
  */
@@ -100,21 +100,6 @@ static MD2_INT S[256]={
 	0xDB, 0x99, 0x8D, 0x33, 0x9F, 0x11, 0x83, 0x14,
 	};
 
-unsigned char *MD2(d, n, md)
-unsigned char *d;
-unsigned long n;
-unsigned char *md;
-	{
-	MD2_CTX c;
-	static unsigned char m[MD2_DIGEST_LENGTH];
-
-	if (md == NULL) md=m;
-	MD2_Init(&c);
-	MD2_Update(&c,d,n);
-	MD2_Final(md,&c);
-	return(md);
-	}
-
 void MD2_Init(c)
 MD2_CTX *c;
 	{
