@@ -48,7 +48,7 @@
 #include <stdio.h>
 #include "md5_locl.h"
 
-char *MD5_version="\0MD5 part of SSLeay v 0.5.1b 23/02/96";
+char *MD5_version="MD5 part of SSLeay 0.5.2 29-Apr-1996";
 
 /* Implemented from RFC1321 The MD5 Message-Digest Algorithm
  */
@@ -64,21 +64,6 @@ static void do_block(MD5_CTX *c, unsigned long *p);
 static void do_block();
 #endif
 
-unsigned char *MD5(d, n, md)
-unsigned char *d;
-unsigned long n;
-unsigned char *md;
-	{
-	MD5_CTX c;
-	static unsigned char m[MD5_DIGEST_LENGTH];
-
-	if (md == NULL) md=m;
-	MD5_Init(&c);
-	MD5_Update(&c,d,n);
-	MD5_Final(md,&c);
-	return(md);
-	}
-
 void MD5_Init(c)
 MD5_CTX *c;
 	{
