@@ -56,7 +56,7 @@
 #include "buffer.h"
 #include "des.h"
 
-char *PEM_version="\0PEM part of SSLeay v 0.5.1b 23/02/96";
+char *PEM_version="PEM part of SSLeay 0.5.2 29-Apr-1996";
 
 #define MIN_LENGTH	4
 
@@ -165,8 +165,14 @@ int (*cb)();
 
 	for (;;)
 		{
-		if (!PEM_read(fp,&nm,&header,&data,&len)) return(0);
-		if (strcmp(nm,name) == 0) break;
+		if (!PEM_read(fp,&nm,&header,&data,&len)) return(NULL);
+		if (	(strcmp(nm,name) == 0) ||
+			((strcmp(nm,PEM_STRING_X509_OLD) == 0) &&
+			(strcmp(name,PEM_STRING_X509) == 0)) ||
+			((strcmp(nm,PEM_STRING_X509_REQ_OLD) == 0) &&
+			(strcmp(name,PEM_STRING_X509_REQ) == 0))
+			)
+			break;
 		free(nm);
 		free(header);
 		free(data);
@@ -306,6 +312,7 @@ int (*callback)();
 	EVP_DecryptInit(&ctx,cipher->cipher,key,&(cipher->iv[0]));
 	EVP_DecryptUpdate(&ctx,data,&i,data,j);
 	o=EVP_DecryptFinal(&ctx,&(data[i]),&j);
+	memset((char *)&ctx,0,sizeof(ctx));
 	j+=i;
 	if (!o)
 		{
