@@ -50,7 +50,7 @@
 #include "sha.h"
 #include "sha_locl.h"
 
-char *SHA1_version="\0SHA1 part of SSLeay v 0.5.1b 23/02/96";
+char *SHA1_version="SHA1 part of SSLeay 0.5.2 29-Apr-1996";
 
 /* Implemented from SHA-1 document - The Secure Hash Algorithm
  */
@@ -72,21 +72,6 @@ static void do_block(SHA_CTX *c, register unsigned long *p);
 static void do_block();
 #endif
 
-unsigned char *SHA1(d, n, md)
-unsigned char *d;
-unsigned long n;
-unsigned char *md;
-	{
-	SHA_CTX c;
-	static unsigned char m[SHA_DIGEST_LENGTH];
-
-	if (md == NULL) md=m;
-	SHA1_Init(&c);
-	SHA1_Update(&c,d,n);
-	SHA1_Final(md,&c);
-	return(md);
-	}
-
 void SHA1_Init(c)
 SHA_CTX *c;
 	{
