@@ -47,18 +47,23 @@
 
 /* Code for stacks
  * Author - Eric Young v 1.0
- * 1.1 eay - Take from netdb and added to SSLeay :-)
+ * 1.1 eay - Take from netdb and added to SSLeay
  *
  * 1.0 eay - First version 29/07/92
  */
 #include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
+#include "crypto.h"
 #include "stack.h"
 
 #define MIN_NODES	4
 
-char *STACK_version="\0STACK part of SSLeay v 0.5.1b 23/02/96";
+char *STACK_version="STACK part of SSLeay 0.5.2 29-Apr-1996";
+
+#ifndef NOPROTO
+#define	FP_ICC	(int (*)(const void *,const void *))
+#else
+#define FP_ICC
+#endif
 
 STACK *sk_new(c)
 int (*c)();
@@ -99,7 +104,7 @@ int loc;
 		st->data=s;
 		st->num_alloc*=2;
 		}
-	if ((loc >= st->num) || (loc < 0))
+	if ((loc >= (int)st->num) || (loc < 0))
 		st->data[st->num]=data;
 	else
 		{
@@ -117,7 +122,7 @@ char *sk_delete_ptr(st,p)
 STACK *st;
 char *p;
 	{
-	int i;
+	unsigned int i;
 
 	for (i=0; i<st->num; i++)
 		if (st->data[i] == p)
@@ -131,10 +136,10 @@ int loc;
 	{
 	char *ret;
 
-	if ((st->num == 0) || (loc < 0) || (loc >= st->num)) return(NULL);
+	if ((st->num == 0) || (loc < 0) || ((unsigned int)loc >= st->num)) return(NULL);
 
 	ret=st->data[loc];
-	if (loc != st->num-1)
+	if ((unsigned int)loc != st->num-1)
 		memcpy( &(st->data[loc]),
 			&(st->data[loc+1]),
 			sizeof(char *)*(st->num-loc-1));
@@ -147,7 +152,7 @@ STACK *st;
 char *data;
 	{
 	char **r;
-	int i;
+	unsigned int i;
 	int (*comp_func)();
 
 	if (st->comp == NULL)
@@ -156,15 +161,15 @@ char *data;
 			if (st->data[i] == data)
 				return(i);
 		}
-	comp_func=st->comp;
+	comp_func=(int (*)())st->comp;
 	if (!st->sorted)
 		{
-		qsort((char *)st->data,st->num,sizeof(char *),comp_func);
+		qsort((char *)st->data,st->num,sizeof(char *),FP_ICC comp_func);
 		st->sorted=1;
 		}
 	if (data == NULL) return(-1);
 	r=(char **)bsearch(&data,(char *)st->data,
-		st->num,sizeof(char *),comp_func);
+		st->num,sizeof(char *),FP_ICC comp_func);
 	if (r == NULL) return(-1);
 	i=(int)(r-st->data);
 	for ( ; i>0; i--)
@@ -208,7 +213,7 @@ void sk_pop_free(st,func)
 STACK *st;
 void (*func)();
 	{
-	int i;
+	unsigned int i;
 
 	if (st == NULL) return;
 	for (i=0; i<st->num; i++)
