@@ -107,6 +107,7 @@ SSL *s;
 			free(buf);
 			s->init_num=0;
 			s->state=ST_SEND_CLIENT_HELLO_A;
+			s->ctx->sess_connect++;
 			BREAK;
 
 		case ST_SEND_CLIENT_HELLO_A:
@@ -122,7 +123,7 @@ SSL *s;
 			ret=get_server_hello(s);
 			if (ret <= 0) return(ret);
 			s->init_num=0;
-			if (!s->hit) /* new connection */
+			if (!s->hit) /* new session */
 				{
 				s->state=ST_SEND_CLIENT_MASTER_KEY_A;
 				BREAK; 
@@ -144,7 +145,7 @@ SSL *s;
 		case ST_START_ENCRYPTION:
 			/* Ok, we now have all the stuff needed to
 			 * start encrypting, so lets fire it up :-) */
-			if (!s->conn->cipher->crypt_init(s,1)) return(-1);
+			if (!s->session->cipher->crypt_init(s,1)) return(-1);
 			s->clear_text=0;
 			s->state=ST_SEND_CLIENT_FINISHED_A;
 			BREAK;
@@ -186,6 +187,7 @@ SSL *s;
 			s->init_buf=NULL;
 			s->init_num=0;
 			ERR_clear_error();
+			s->ctx->sess_connect_good++;
 			return(1);
 		default:
 			SSLerr(SSL_F_SSL_CONNECT,SSL_R_UNKNOWN_STATE);
@@ -257,9 +259,9 @@ SSL *s;
 		}
 	else
 		{
-		if (s->conn->session_id != NULL) free(s->conn->session_id);
-		s->conn->session_id=NULL;
-		s->conn->session_id_length=0;
+		if (s->session->session_id != NULL) free(s->session->session_id);
+		s->session->session_id=NULL;
+		s->session->session_id_length=0;
 
 		if (ssl_set_certificate(s,s->state_cert_type,
 			s->state_cert_length,p) <= 0)
@@ -278,12 +280,12 @@ SSL *s;
 
 		/* what we will do is copy the ciphers we are happy to
 		 * use to a temp area and then check the server ones
-		 * are put those that are in the conn structure again. */
-		cipher= &(s->conn->ciphers[0]);
+		 * are put those that are in the session structure again. */
+		cipher= &(s->session->ciphers[0]);
 		memcpy(&(tmp_c[0]),cipher,sizeof(CIPHER *)*NUM_CIPHERS);
-		num_c=s->conn->num_ciphers;
+		num_c=s->session->num_ciphers;
 		nnum_c=0;
-		for (i=0; i<s->state_csl; i+=3)
+		for (i=0; i<(int)s->state_csl; i+=3)
 			{
 			CIPHER c,*cp= &c,**cpp;
 
@@ -296,12 +298,12 @@ SSL *s;
 				(int (*)())ssl_cipher_ptr_cmp);
 			if ((cpp != NULL) && ((*cpp)->valid))
 				{
-				s->conn->ciphers[nnum_c++]= *cpp;
+				s->session->ciphers[nnum_c++]= *cpp;
 				}
 			if (nnum_c == NUM_CIPHERS) break;
 			}
 		p+=s->state_csl;
-		s->conn->num_ciphers=nnum_c;
+		s->session->num_ciphers=nnum_c;
 
 		if (nnum_c == 0)
 			{
@@ -310,9 +312,9 @@ SSL *s;
 			return(-1);
 			}
 		qsort((char *)&(cipher[0]),nnum_c,sizeof(CIPHER *),
-			(int (*)())ssl_cipher_ptr_cmp);
+			(int (*)(P_CC_CC))ssl_cipher_ptr_cmp);
 
-		/* we now have the sorted cipher in s->conn->ciphers; */
+		/* we now have the sorted cipher in s->session->ciphers; */
 
 		/* pick a cipher - which one? */
 		if (!choose_cipher(s))
@@ -355,15 +357,15 @@ SSL *s;
 	buf=s->init_buf;
 	if (s->state == ST_SEND_CLIENT_HELLO_A)
 		{
-		if (s->conn == NULL)
-			if (!ssl_new_conn(s,0))
+		if (s->session == NULL)
+			if (!ssl_get_new_session(s,0))
 				{
 				ssl_return_error(s);
 				return(-1);
 				}
 		/* else use the pre-loaded session */
 
-		cipher= &(s->conn->ciphers[0]);
+		cipher= &(s->session->ciphers[0]);
 		p=buf;					/* header */
 		d=p+9;					/* data section */
 		*(p++)=SSL_MT_CLIENT_HELLO;		/* type */
@@ -380,14 +382,14 @@ SSL *s;
 				n+=3;
 				}
 			}
-		s->conn->num_ciphers=j;
+		s->session->num_ciphers=j;
 		s2n(n,p);			/* cipher spec num bytes */
 
-		if (s->conn->session_id != NULL)
+		if (s->session->session_id != NULL)
 			{
-			i=s->conn->session_id_length;
+			i=s->session->session_id_length;
 			s2n(i,p);		/* session id length */
-			memcpy(d,s->conn->session_id,(unsigned int)i);
+			memcpy(d,s->session->session_id,(unsigned int)i);
 			d+=i;
 			}
 		else
@@ -436,15 +438,15 @@ SSL *s;
 		p=buf;
 		d=p+10;
 		*(p++)=SSL_MT_CLIENT_MASTER_KEY;/* type */
-		*(p++)=s->conn->cipher->c1;	/* cipher type - byte 1 */
-		*(p++)=s->conn->cipher->c2;	/* cipher type - byte 2 */
-		*(p++)=s->conn->cipher->c3;	/* cipher type - byte 3 */
+		*(p++)=s->session->cipher->c1;	/* cipher type - byte 1 */
+		*(p++)=s->session->cipher->c2;	/* cipher type - byte 2 */
+		*(p++)=s->session->cipher->c3;	/* cipher type - byte 3 */
 
 		/* make a master key */
-		i=s->conn->master_key_length=(unsigned char)
-			s->conn->cipher->key_size;
-		s->conn->master_key=(unsigned char *)malloc((unsigned int)i+1);
-		if (s->conn->master_key == NULL)
+		i=s->session->master_key_length=(unsigned char)
+			s->session->cipher->key_size;
+		s->session->master_key=(unsigned char *)malloc((unsigned int)i+1);
+		if (s->session->master_key == NULL)
 			{
 			ssl_return_error(s);
 			SSLerr(SSL_F_CLIENT_MASTER_KEY,ERR_R_MALLOC_FAILURE);
@@ -452,25 +454,25 @@ SSL *s;
 			}
 		if (s->ctx->reverse)
 			{
-			if (s->ctx->master_key_length != i)
+			if ((int)s->ctx->master_key_length != i)
 				{
 				SSLerr(SSL_F_CLIENT_MASTER_KEY,SSL_R_REVERSE_MASTER_KEY_LENGTH_IS_WRONG);
 				return(-1);
 				}
-			memcpy(s->conn->master_key,s->ctx->master_key,i);
+			memcpy(s->session->master_key,s->ctx->master_key,i);
 			}
 		else
-			RAND_bytes(s->conn->master_key,i);
+			RAND_bytes(s->session->master_key,i);
 
 		/* make key_arg data */
-		i=s->conn->key_arg_length=s->conn->cipher->key_arg_size;
+		i=s->session->key_arg_length=s->session->cipher->key_arg_size;
 		if (i == 0)
-			s->conn->key_arg=NULL;
+			s->session->key_arg=NULL;
 		else
 			{
-			s->conn->key_arg=(unsigned char *)
+			s->session->key_arg=(unsigned char *)
 				malloc((unsigned int)i);
-			if (s->conn->key_arg == NULL)
+			if (s->session->key_arg == NULL)
 				{
 				ssl_return_error(s);
 				SSLerr(SSL_F_CLIENT_MASTER_KEY,ERR_R_MALLOC_FAILURE);
@@ -478,19 +480,19 @@ SSL *s;
 				}
 			if (s->ctx->reverse)
 				{
-				if (s->ctx->key_arg_length != i)
+				if ((int)s->ctx->key_arg_length != i)
 					{
 					SSLerr(SSL_F_CLIENT_MASTER_KEY,SSL_R_REVERSE_KEY_ARG_LENGTH_IS_WRONG);
 					return(-1);
 					}
-				memcpy(s->conn->key_arg,s->ctx->key_arg,i);
+				memcpy(s->session->key_arg,s->ctx->key_arg,i);
 				}
 			else
-				RAND_bytes(s->conn->key_arg,i);
+				RAND_bytes(s->session->key_arg,i);
 			}
 
-		i=s->conn->cipher->key_size*8;
-		enc=s->conn->cipher->enc_bits;
+		i=s->session->cipher->key_size*8;
+		enc=s->session->cipher->enc_bits;
 		if (enc == 0) enc=i;
 		if (i < enc)
 			{
@@ -502,12 +504,12 @@ SSL *s;
 		/* bytes or bits? */
 		clear/=8; /* clear */
 		s2n(clear,p);
-		memcpy(d,s->conn->master_key,(unsigned int)clear);
+		memcpy(d,s->session->master_key,(unsigned int)clear);
 		d+=clear;
 
 		enc/=8;
-		enc=s->conn->cert->public_encrypt(s->conn->cert,enc,
-			&(s->conn->master_key[clear]),d);
+		enc=s->session->cert->public_encrypt(s->session->cert,enc,
+			&(s->session->master_key[clear]),d);
 		if (enc <= 0)
 			{
 			ssl_return_error(s);
@@ -516,9 +518,9 @@ SSL *s;
 			}
 		s2n(enc,p);
 		d+=enc;
-		karg=s->conn->cipher->key_arg_size;	
+		karg=s->session->cipher->key_arg_size;	
 		s2n(karg,p); /* key arg size */
-		memcpy(d,s->conn->key_arg,(unsigned int)karg);
+		memcpy(d,s->session->key_arg,(unsigned int)karg);
 		d+=karg;
 
 		s->state=ST_SEND_CLIENT_MASTER_KEY_B;
@@ -605,10 +607,10 @@ SSL *s;
 		 * do it first so we can reuse buf :-) */
 		p=buf;
 		EVP_SignInit(&ctx,EVP_md5);
-		EVP_SignUpdate(&ctx,s->conn->key_material,
-			(unsigned int)s->conn->key_material_length);
+		EVP_SignUpdate(&ctx,s->key_material,
+			(unsigned int)s->key_material_length);
 		EVP_SignUpdate(&ctx,cert_ch,(unsigned int)cert_ch_len);
-		n=i2d_X509(s->conn->cert->x509,&p);
+		n=i2d_X509(s->session->cert->x509,&p);
 		EVP_SignUpdate(&ctx,buf,(unsigned int)n);
 
 		p=buf;
@@ -625,6 +627,7 @@ SSL *s;
 			 * We will contiune with a 0 length signature
 			 */
 			}
+		memset(&ctx,0,sizeof(ctx));
 		s2n(n,p);
 		d+=n;
 
@@ -669,7 +672,7 @@ SSL *s;
 	p=s->init_buf;
 	i=SSL_read(s,(char *)&(p[s->init_num]),
 		(unsigned int)s->challenge_length-s->init_num);
-	if (i < (s->challenge_length-s->init_num))
+	if (i < ((int)s->challenge_length-s->init_num))
 		return(ssl_part_read(s,SSL_F_GET_SERVER_VERIFY,i));
 	if (memcmp(p,s->challenge,(unsigned int)s->challenge_length) != 0)
 		{
@@ -716,21 +719,21 @@ SSL *s;
 		}
 
 	i=SSL_read(s,(char *)&(buf[s->init_num]),
-		SESSION_ID_LENGTH-s->init_num);
-	if (i < (SESSION_ID_LENGTH-s->init_num))
+		SSL_SESSION_ID_LENGTH-s->init_num);
+	if (i < (SSL_SESSION_ID_LENGTH-s->init_num))
 		return(ssl_part_read(s,SSL_F_GET_SERVER_FINISHED,i));
 
-	if (!s->hit) /* new connection */
+	if (!s->hit) /* new session */
 		{
-		if ((s->conn->session_id == NULL) ||
-			(s->conn->session_id_length <
-				SESSION_ID_LENGTH))
+		if ((s->session->session_id == NULL) ||
+			(s->session->session_id_length <
+				SSL_SESSION_ID_LENGTH))
 			{
-			if (s->conn->session_id)
-				free(s->conn->session_id);
-			s->conn->session_id=(unsigned char *)
-				malloc(SESSION_ID_LENGTH);
-			if (s->conn->session_id == NULL)
+			if (s->session->session_id)
+				free(s->session->session_id);
+			s->session->session_id=(unsigned char *)
+				malloc(SSL_SESSION_ID_LENGTH);
+			if (s->session->session_id == NULL)
 				{
 				ssl_return_error(s);
 				SSLerr(SSL_F_GET_SERVER_FINISHED,
@@ -738,17 +741,17 @@ SSL *s;
 				return(-1);
 				}
 			}
-		s->conn->session_id_length=SESSION_ID_LENGTH;
-		memcpy(s->conn->session_id,p,SESSION_ID_LENGTH);
-		ssl_add_hash_conn(s,s->conn); /* should return error? */
+		s->session->session_id_length=SSL_SESSION_ID_LENGTH;
+		memcpy(s->session->session_id,p,SSL_SESSION_ID_LENGTH);
+		SSL_add_session(s->ctx,s->session); /* should return error? */
 		}
 	else
 		{
-		if (memcmp(buf,s->conn->session_id,
-			(unsigned int)s->conn->session_id_length) != 0)
+		if (memcmp(buf,s->session->session_id,
+			(unsigned int)s->session->session_id_length) != 0)
 			{
 			ssl_return_error(s);
-			SSLerr(SSL_F_GET_SERVER_FINISHED,SSL_R_SESSION_ID_IS_DIFFERENT);
+			SSLerr(SSL_F_GET_SERVER_FINISHED,SSL_R_SSL_SESSION_ID_IS_DIFFERENT);
 			return(-1);
 			}
 		}
@@ -758,39 +761,39 @@ SSL *s;
 static int choose_cipher(s)
 SSL *s;
 	{
-	int j,i=0;
+	int j,i;
 	char *p;
 
 	if (s->ctx->reverse)
 		{
 		for (;;)
 			{
-			for (j=0; j < s->conn->num_ciphers; j++)
+			for (j=0; j < s->session->num_ciphers; j++)
 				{
 				if (strcmp(s->ctx->cipher,
-					s->conn->ciphers[j]->name) == 0)
+					s->session->ciphers[j]->name) == 0)
 					{
-					s->conn->cipher=s->conn->ciphers[j];
+					s->session->cipher=s->session->ciphers[j];
 					return(1);
 					}
 				}
 			return(0);
 			}
 		}
-
+	i=0;
 	for (;;)
 		{
-		p=SSL_get_pref_cipher(s,i++);
+		p=SSL_get_cipher_list(s,i++);
 		if (p == NULL)
 			{
-			s->conn->cipher=NULL;
+			s->session->cipher=NULL;
 			return(0);
 			}
-		for (j=0; j < s->conn->num_ciphers; j++)
+		for (j=0; j < s->session->num_ciphers; j++)
 			{
-			if (strcmp(p,s->conn->ciphers[j]->name) == 0)
+			if (strcmp(p,s->session->ciphers[j]->name) == 0)
 				{
-				s->conn->cipher=s->conn->ciphers[j];
+				s->session->cipher=s->session->ciphers[j];
 				return(1);
 				}
 			}
