@@ -55,13 +55,15 @@ static ERR_STRING_DATA SSL_str_functs[]=
 {ERR_PACK(0,SSL_F_CLIENT_CERTIFICATE,0),	"CLIENT_CERTIFICATE"},
 {ERR_PACK(0,SSL_F_CLIENT_HELLO,0),	"CLIENT_HELLO"},
 {ERR_PACK(0,SSL_F_CLIENT_MASTER_KEY,0),	"CLIENT_MASTER_KEY"},
+{ERR_PACK(0,SSL_F_D2I_SSL_SESSION,0),	"d2i_SSL_SESSION"},
 {ERR_PACK(0,SSL_F_GET_CLIENT_FINISHED,0),	"GET_CLIENT_FINISHED"},
 {ERR_PACK(0,SSL_F_GET_CLIENT_HELLO,0),	"GET_CLIENT_HELLO"},
 {ERR_PACK(0,SSL_F_GET_CLIENT_MASTER_KEY,0),	"GET_CLIENT_MASTER_KEY"},
-{ERR_PACK(0,SSL_F_GET_NEW_CONN,0),	"GET_NEW_CONN"},
+{ERR_PACK(0,SSL_F_GET_NEW_SSL_SESSION,0),	"GET_NEW_SSL_SESSION"},
 {ERR_PACK(0,SSL_F_GET_SERVER_FINISHED,0),	"GET_SERVER_FINISHED"},
 {ERR_PACK(0,SSL_F_GET_SERVER_HELLO,0),	"GET_SERVER_HELLO"},
 {ERR_PACK(0,SSL_F_GET_SERVER_VERIFY,0),	"GET_SERVER_VERIFY"},
+{ERR_PACK(0,SSL_F_I2D_SSL_SESSION,0),	"i2d_SSL_SESSION"},
 {ERR_PACK(0,SSL_F_REQUEST_CERTIFICATE,0),	"REQUEST_CERTIFICATE"},
 {ERR_PACK(0,SSL_F_SERVER_HELLO,0),	"SERVER_HELLO"},
 {ERR_PACK(0,SSL_F_SSL_ACCEPT,0),	"SSL_accept"},
@@ -75,9 +77,11 @@ static ERR_STRING_DATA SSL_str_functs[]=
 {ERR_PACK(0,SSL_F_SSL_ENC_DES_EDE3_CBC_INIT,0),	"SSL_ENC_DES_EDE3_CBC_INIT"},
 {ERR_PACK(0,SSL_F_SSL_ENC_IDEA_CBC_INIT,0),	"SSL_ENC_IDEA_CBC_INIT"},
 {ERR_PACK(0,SSL_F_SSL_ENC_NULL_INIT,0),	"SSL_ENC_NULL_INIT"},
+{ERR_PACK(0,SSL_F_SSL_ENC_RC2_CBC_INIT,0),	"SSL_ENC_RC2_CBC_INIT"},
 {ERR_PACK(0,SSL_F_SSL_ENC_RC4_INIT,0),	"SSL_ENC_RC4_INIT"},
 {ERR_PACK(0,SSL_F_SSL_NEW,0),	"SSL_new"},
 {ERR_PACK(0,SSL_F_SSL_READ,0),	"SSL_read"},
+{ERR_PACK(0,SSL_F_SSL_SESSION_NEW,0),	"SSL_SESSION_new"},
 {ERR_PACK(0,SSL_F_SSL_SET_CERTIFICATE,0),	"SSL_SET_CERTIFICATE"},
 {ERR_PACK(0,SSL_F_SSL_USE_CERTIFICATE_ASN1,0),	"SSL_use_certificate_ASN1"},
 {ERR_PACK(0,SSL_F_SSL_USE_CERTIFICATE_FILE,0),	"SSL_use_certificate_file"},
@@ -94,13 +98,14 @@ static ERR_STRING_DATA SSL_str_reasons[]=
 {SSL_R_BAD_CHECKSUM                      ,"bad checksum"},
 {SSL_R_BAD_MAC_DECODE                    ,"bad mac decode"},
 {SSL_R_BAD_RESPONSE_ARGUMENT             ,"bad response argument"},
-{SSL_R_BAD_SESSION_ID_LENGTH             ,"bad session id length"},
 {SSL_R_BAD_SSL_FILETYPE                  ,"bad ssl filetype"},
+{SSL_R_BAD_SSL_SESSION_ID_LENGTH         ,"bad ssl session id length"},
 {SSL_R_BAD_STATE                         ,"bad state"},
 {SSL_R_BAD_WRITE_RETRY                   ,"bad write retry"},
 {SSL_R_CHALLENGE_IS_DIFFERENT            ,"challenge is different"},
+{SSL_R_CIPHER_CODE_TOO_LONG              ,"cipher code too long"},
 {SSL_R_CIPHER_TABLE_SRC_ERROR            ,"cipher table src error"},
-{SSL_R_CONNECTION_ID_IS_DIFFERENT        ,"connection id is different"},
+{SSL_R_CONECTION_ID_IS_DIFFERENT         ,"conection id is different"},
 {SSL_R_INVALID_CHALLENGE_LENGTH          ,"invalid challenge length"},
 {SSL_R_NO_CERTIFICATE_SET                ,"no certificate set"},
 {SSL_R_NO_CERTIFICATE_SPECIFIED          ,"no certificate specified"},
@@ -118,13 +123,14 @@ static ERR_STRING_DATA SSL_str_reasons[]=
 {SSL_R_PERR_ERROR_NO_CERTIFICATE         ,"perr error no certificate"},
 {SSL_R_PUBLIC_KEY_ENCRYPT_ERROR          ,"public key encrypt error"},
 {SSL_R_READ_WRONG_PACKET_TYPE            ,"read wrong packet type"},
-{SSL_R_REVERSE_CONN_ID_LENGTH_IS_WRONG   ,"reverse conn id length is wrong"},
 {SSL_R_REVERSE_KEY_ARG_LENGTH_IS_WRONG   ,"reverse key arg length is wrong"},
 {SSL_R_REVERSE_MASTER_KEY_LENGTH_IS_WRONG,"reverse master key length is wrong"},
-{SSL_R_SESSION_ID_IS_DIFFERENT           ,"session id is different"},
+{SSL_R_REVERSE_SSL_SESSION_ID_LENGTH_IS_WRONG,"reverse ssl session id length is wrong"},
 {SSL_R_SHORT_READ                        ,"short read"},
+{SSL_R_SSL_SESSION_ID_IS_DIFFERENT       ,"ssl session id is different"},
 {SSL_R_UNKNOWN_REMOTE_ERROR_TYPE         ,"unknown remote error type"},
 {SSL_R_UNKNOWN_STATE                     ,"unknown state"},
+{SSL_R_UNSUPORTED_CIPHER                 ,"unsuported cipher"},
 {0,NULL},
 	};
 
