@@ -46,16 +46,18 @@
  */
 
 #include <stdio.h>
+#include "md5.h"
 #include "ssl_locl.h"
 #include "ssl_des.h"
 #include "ssl_md5.h"
 #include "ssl_sha.h"
 #include "ssl_rc4.h"
+#include "ssl_rc2.h"
 #include "ssl_idea.h"
 #include "ssl_rsa.h"
 #include "ssl_null.h"
 
-char *SSL_version="\0SSLeay v 0.5.1b 23/02/96";
+char *SSL_version="SSLeay 0.5.2 29-Apr-1996";
 
 /* THIS ARRAY MUST BE KEPT ORDERED BY c1, c2 and c3.
  * basically the second last 'value' which is a #define for these 3
@@ -75,27 +77,26 @@ CIPHER ssl_ciphers[NUM_CIPHERS]={
 /* RC4_128_EXPORT40_WITH_MD5 */
 	{	1,SSL_TXT_RC4_128_EXPORT40_WITH_MD5,
 	ssl_enc_rc4_init,ssl_enc_rc4,ssl_compute_md5_mac,
-	16,MD5_MAC_SIZE,1,0,
-	SSL_CK_RC4_128_EXPORT40_WITH_MD5,40},
+	16,MD5_MAC_SIZE,1,0, SSL_CK_RC4_128_EXPORT40_WITH_MD5,40},
 #endif
 
+#ifndef NORC2
 /* RC2_128_CBC_WITH_MD5 */
-	{	0,SSL_TXT_RC2_128_CBC_WITH_MD5,
-	NULL,NULL,NULL,
-	0,0,0,0 ,SSL_CK_RC2_128_CBC_WITH_MD5,0},
+	{	1,SSL_TXT_RC2_128_CBC_WITH_MD5,
+	ssl_enc_rc2_cbc_init,ssl_enc_rc2_cbc,ssl_compute_md5_mac,
+	16,MD5_MAC_SIZE,8,8,SSL_CK_RC2_128_CBC_WITH_MD5,0},
 
 /* RC2_128_CBC_EXPORT40_WITH_MD5 */
-	{	0,SSL_TXT_RC2_128_CBC_EXPORT40_WITH_MD5,
-	NULL,NULL,NULL,
-	0,0,0,0,
-	SSL_CK_RC2_128_CBC_EXPORT40_WITH_MD5,40},
+	{	1,SSL_TXT_RC2_128_CBC_EXPORT40_WITH_MD5,
+	ssl_enc_rc2_cbc_init,ssl_enc_rc2_cbc,ssl_compute_md5_mac,
+	16,MD5_MAC_SIZE,8,8,SSL_CK_RC2_128_CBC_EXPORT40_WITH_MD5,40},
+#endif
 
 #ifndef NOIDEA
 /* IDEA_128_CBC_WITH_MD5 */
 	{	1,SSL_TXT_IDEA_128_CBC_WITH_MD5,
 	ssl_enc_idea_cbc_init,ssl_enc_idea_cbc,ssl_compute_md5_mac,
-	16,MD5_MAC_SIZE,8,8,
-	SSL_CK_IDEA_128_CBC_WITH_MD5,0},
+	16,MD5_MAC_SIZE,8,8, SSL_CK_IDEA_128_CBC_WITH_MD5,0},
 #endif
 
 /* DES_64_CBC_WITH_MD5 */
@@ -129,28 +130,33 @@ CIPHER ssl_ciphers[NUM_CIPHERS]={
 	SSL_CK_DES_64_CFB64_WITH_MD5_1,0},
 
 /* NULL SSLeay (testing) */
-	{	1,SSL_TXT_NULL,
+	{	0,SSL_TXT_NULL,
 	ssl_enc_null_init,ssl_enc_null,ssl_compute_null_mac,
 	0,0,1,0,SSL_CK_NULL,0},
 
 /* end of list :-) */
 	};
 
-static int num_pref_cipher=8;
+static int num_pref_cipher=(4 HAVE_IDEA HAVE_RC4 HAVE_RC2);
 static char *pref_cipher[]={
-	SSL_TXT_DES_64_CBC_WITH_MD5,		/* Small key */
-	SSL_TXT_DES_64_CBC_WITH_SHA,		/* Small key, SHA */
 #ifndef NORC4
 	SSL_TXT_RC4_128_WITH_MD5,		/* Stream cipher but fast */
 #endif
-	SSL_TXT_DES_192_EDE3_CBC_WITH_MD5,	/* paranoid but slow */
-	SSL_TXT_DES_192_EDE3_CBC_WITH_SHA,	/* paranoid but slow, SHA */
 #ifndef NOIDEA
 	SSL_TXT_IDEA_128_CBC_WITH_MD5,		/* best? */
 #endif
-/*	SSL_TXT_DES_64_CFB64_WITH_MD5_1,*/	/* 1 byte MAC, small key */
+#ifndef NORC2
+	SSL_TXT_RC2_128_CBC_WITH_MD5,		/* how good is RC2? */
+#endif
+	SSL_TXT_DES_64_CBC_WITH_MD5,		/* Small key */
+	SSL_TXT_DES_64_CBC_WITH_SHA,		/* Small key, SHA */
+	SSL_TXT_DES_192_EDE3_CBC_WITH_MD5,	/* paranoid but slow */
+	SSL_TXT_DES_192_EDE3_CBC_WITH_SHA,	/* paranoid but slow, SHA */
 #ifndef NORC4
 	SSL_TXT_RC4_128_EXPORT40_WITH_MD5,	/* Smaller key easy to break */
+#endif
+#ifndef NORC2
+	SSL_TXT_RC2_128_CBC_EXPORT40_WITH_MD5,/* Slow and Small key :-( */
 #endif
 	NULL
 	};
@@ -190,6 +196,7 @@ void SSL_clear(s)
 SSL *s;
 	{
 	s->version=SSL_SERVER_VERSION;
+	s->rwstate=SSL_NOTHING;
 	s->state=ST_BEFORE;
 	s->rstate=ST_READ_HEADER;
 	if (s->init_buf != NULL) free(s->init_buf);
@@ -198,6 +205,7 @@ SSL *s;
 	if (s->state_ccl != NULL) free(s->state_ccl);
 	s->state_ccl=NULL;
 	s->read_ahead=0;
+	s->wnum=0;
 	s->wpend_tot=0;
 	s->wpend_off=0;
 	s->wpend_len=0;
@@ -228,10 +236,15 @@ SSL *s;
 	s->challenge_length=0;
 	if (s->challenge != NULL) free(s->challenge);
 	s->challenge=NULL;
+
 	s->conn_id_length=0;
 	if (s->conn_id != NULL) free(s->conn_id);
 	s->conn_id=NULL;
 
+	s->key_material_length=0;
+	if (s->key_material != NULL) free(s->key_material);
+	s->key_material=NULL;
+
 	s->send=0;
 	s->clear_text=1;
 	s->hit=0;
@@ -240,8 +253,6 @@ SSL *s;
 	s->read_sequence=0;
 	s->write_sequence=0;
 	s->trust_level=0;
-
-	s->ctx->references++;
 	}
 
 SSL *SSL_new(ctx)
@@ -265,19 +276,22 @@ SSL_CTX *ctx;
 	if (s->rbuf == NULL) goto err;
 	s->wbuf=(unsigned char *)malloc(SSL_MAX_RECORD_LENGTH_2_BYTE_HEADER+2);
 	if (s->wbuf == NULL) goto err;
-	s->num_pref_cipher=0;
-	s->pref_cipher=NULL;
+	s->num_cipher_list=0;
+	s->cipher_list=NULL;
 	s->crypt_state=NULL;
 	s->challenge=NULL;
 	s->conn_id=NULL;
-	s->conn=NULL;
+	s->key_material=NULL;
+	s->session=NULL;
 	s->cert=NULL;
 	s->peer=NULL;
 	s->peer_status=SSL_PEER_NOT_SET;
 	s->verify_mode=SSL_VERIFY_NONE;
 	s->verify_callback=NULL;
 	s->ctx=ctx;
+	CS_BEGIN;
 	ctx->references++;
+	CS_END;
 	s->state_ccl=NULL;
 
 	SSL_clear(s);
@@ -293,29 +307,25 @@ SSL *s;
 	/* add extra stuff */
 	free(s->rbuf);
 	free(s->wbuf);
+	if (s->cipher_list != NULL)
+		{
+		free(s->cipher_list[0]);
+		free(s->cipher_list);
+		}
 	if (s->init_buf != NULL) free(s->init_buf);
 	if (s->crypt_state != NULL) free(s->crypt_state);
 	/* again, just let them linger around */
-	if (s->conn != NULL) ssl_conn_free(s,s->conn);
+	if (s->session != NULL) SSL_SESSION_free(s->session);
 	if (s->cert != NULL) ssl_cert_free(s->cert);
 	if (s->challenge != NULL) free(s->challenge);
 	if (s->conn_id != NULL) free(s->conn_id);
+	if (s->key_material != NULL) free(s->key_material);
 	if (s->state_ccl != NULL) free(s->state_ccl);
 	/* free up if allocated */
 	if ((s->peer_status == SSL_PEER_IN_SSL) && (s->peer != NULL))
 		X509_free(s->peer);
 
-	/* TJH - Eric the following is just plain *wrong*
-	 *       the create is separate so the free must
-	 *       be separate too ... otherwise things get
-	 *       smashed!
-	 *     - what you really wanted to do I think was
-	 *       decrement the reference counter perhaps
-	 *       which is what I've added
-	 */
-	/* if (s->ctx) SSL_CTX_free(s->ctx); /**/
-	if (s->ctx) 
-	    s->ctx->references--;
+	if (s->ctx) SSL_CTX_free(s->ctx);
 	free((char *)s);
 	}
 
@@ -348,9 +358,9 @@ char *b;
 void ssl_return_error(s)
 SSL *s;
 	{
-	static unsigned char buf[1]={SSL_MT_ERROR};
+	static char buf[1]={SSL_MT_ERROR};
 
-	SSL_write(s,(char *)buf,1);
+	SSL_write(s,buf,1);
 #ifdef ABORT_DEBUG
 	abort(1);
 #endif
@@ -371,25 +381,24 @@ SSL *s;
 	return(s->rfd);
 	}
 
-SSL_CTX *SSL_get_SSL_CTX(s)
+int SSL_set_cipher_list(s, str)
 SSL *s;
+char *str;
 	{
-	return(s->ctx);
+	return(ssl_make_cipher_list(&s->cipher_list,&s->num_cipher_list,str));
 	}
 
-int SSL_set_pref_cipher(s, str)
-SSL *s;
+int ssl_make_cipher_list(rp,n,str)
+char ***rp;
+int *n;
 char *str;
 	{
 	int i,j;
 	char *p,**pp;
 
 	if (str == NULL) return(1);
-	if (s->pref_cipher != NULL)
-		{
-		free(s->pref_cipher[0]);
-		free(s->pref_cipher);
-		}
+	if (p == NULL) return(0);
+	if (n == NULL) return(0);
 
 	p=str;
 	i=(*p == '\0')?0:1;
@@ -398,7 +407,7 @@ char *str;
 	pp=(char **)malloc(sizeof(char *)*(i+1));
 	if (pp == NULL) goto err;
 	pp[0]=(char *)malloc(strlen(str)+1);
-	if (pp == NULL) goto err;
+	if (pp[0] == NULL) goto err;
 	strcpy(pp[0],str);
 	p=pp[0];
 	j=1;
@@ -412,34 +421,47 @@ char *str;
 		pp[j++]=p;
 		}
 	pp[j]=NULL;
-	s->num_pref_cipher=i;
-	s->pref_cipher=pp;
+	if (*rp != NULL)
+		{
+		free(*rp[0]);
+		free(*rp);
+		}
+
+	*rp=pp;
+	*n=i;
 	return(1);
 err:
 	SSLerr(SSL_F_SSL_NEW,ERR_R_MALLOC_FAILURE);
 	return(0);
 	}
 
-char *SSL_get_pref_cipher(s, n)
+char *SSL_get_cipher_list(s, n)
 SSL *s;
 int n;
 	{
-	if (n < s->num_pref_cipher)
-		return(s->pref_cipher[n]);
-
-	if (s->num_pref_cipher != 0) return(NULL);
-
-	n-=s->num_pref_cipher;
-	if (n < num_pref_cipher)
-		return(pref_cipher[n]);
+	if (s->num_cipher_list > 0)
+		{
+		if (n < s->num_cipher_list)
+			return(s->cipher_list[n]);
+		}
+	else if (s->ctx->num_cipher_list > 0)
+		{
+		if (n < s->ctx->num_cipher_list)
+			return(s->ctx->cipher_list[n]);
+		}
+	else
+		{
+		if (n < num_pref_cipher)
+			return(pref_cipher[n]);
+		}
 	return(NULL);
 	}
 
 char *SSL_get_cipher(s)
 SSL *s;
 	{
-	if ((s->conn != NULL) && (s->conn->cipher != NULL))
-		return(s->conn->cipher->name);
+	if ((s->session != NULL) && (s->session->cipher != NULL))
+		return(s->session->cipher->name);
 	return(NULL);
 	}
 
@@ -452,15 +474,15 @@ int len;
 	CIPHER **c;
 	int i;
 
-	if ((s->conn == NULL) || (s->conn->ciphers == NULL) ||
+	if ((s->session == NULL) || (s->session->ciphers == NULL) ||
 		(len < 2))
 		return(NULL);
 
 	p=buf;
-	c= &(s->conn->ciphers[0]);
+	c= &(s->session->ciphers[0]);
 	len--;
 	i=0;
-	for (i=0; i<s->conn->num_ciphers; i++)
+	for (i=0; i<s->session->num_ciphers; i++)
 		{
 		for (cp=c[i]->name; *cp; )
 			{
@@ -478,16 +500,6 @@ int len;
 	return(buf);
 	}
 
-int SSL_copy_session_id(to, from)
-SSL *to;
-SSL *from;
-	{
-	if (from->conn == NULL) return(0);
-	to->conn=from->conn;
-	from->conn->references++;
-	return(1);
-	}
-
 void SSL_set_verify(s, mode, callback)
 SSL *s;
 int mode;
@@ -516,34 +528,6 @@ SSL *s;
 	return(s->rbuf_left);
 	}
 
-int SSL_set_timeout(s, t)
-SSL *s;
-int t;
-	{
-	if (s->conn == NULL) return(0);
-	s->conn->timeout=t;
-	return(1);
-	}
-
-long SSL_get_timeout(s)
-SSL *s;
-	{
-	if (s->conn == NULL) return(0);
-	return(s->conn->timeout);
-	}
-
-long SSL_get_time(s)
-SSL *s;
-	{
-	if (s->conn == NULL) return(0);
-	return(s->conn->time);
-	}
-
-void SSL_flush_connections()
-	{
-	/* not implemented yet */
-	}
-
 int SSL_is_init_finished(s)
 SSL *s;
 	{
@@ -560,12 +544,14 @@ SSL *s;
 	else if (s->peer_status == SSL_PEER_IN_SSL)
 		r=s->peer;
 	else if ((s->peer_status == SSL_PEER_IN_CERT) &&
-		(s->conn != NULL) &&
-		(s->conn->cert != NULL))
-		r=s->conn->cert->x509;
+		(s->session != NULL) &&
+		(s->session->cert != NULL))
+		r=s->session->cert->x509;
 
+	CS_BEGIN;
 	if (r == NULL) return(0);
 	r->references++;
+	CS_END;
 	return(r);
 	}
 
@@ -593,3 +579,26 @@ CIPHER **ap,**bp;
 	if (i) return(i);
 	return(a->c3-b->c3);
 	}
+
+void ssl_generate_key_material(s)
+SSL *s;
+	{
+	unsigned int i;
+	MD5_CTX ctx;
+	unsigned char *km;
+	unsigned char c='0';
+
+	km=s->key_material;
+	for (i=0; i<s->key_material_length; i+=MD5_DIGEST_LENGTH)
+		{
+		MD5_Init(&ctx);
+
+		MD5_Update(&ctx,s->session->master_key,s->session->master_key_length);
+		MD5_Update(&ctx,(unsigned char *)&c,1);
+		c++;
+		MD5_Update(&ctx,s->challenge,s->challenge_length);
+		MD5_Update(&ctx,s->conn_id,s->conn_id_length);
+		MD5_Final(km,&ctx);
+		km+=MD5_DIGEST_LENGTH;
+		}
+	}
